DAT objectName          byte "HexEyesX150131a", 0
CON
{{
  ******* Public Notes *******

  Author: Duane Degn
  Date: December 25, 2013
  Updated" January 27, 2014
  
  MIT License

  An attempt to control an 8x8 LED array with a MAX7219 chip.

  The number of arrays used should be set from the parent object.

  This object uses an array of bytes to store "pixels".

  Since the size of the array is dependent on the number of
  LED displays used, the location of this pixel buffer needs to be
  set from the parent object.

  The "Init" method should be called before using other methods.

  Many of the parameters may be changed on the fly.

  The parameter "rotatedFlag_" is used to indicate the LED displays
  have been rotated 90 degrees. When a rotated display is used the
  pixel locations are arranged differently in memory.  

  The constants marked with "User changeable" may be
  changed based on the LED project is configured.

  The code in this program is pretty much all original.
  I did look at Hover1's 7-segment MAX7219 code. Which
  helped get me started.
 
}}
{
  ******* Private Notes *******

   131225c Displays a message one character at a time.
   25c Appears to be working.
   25d Change font with each message.
   25e Try to rotate text.
   25e Didn't work.
   25g Rotation worked.
   25h Try to add scrolling.
   25j Working reasonably well.
   25k Seems to scroll single rotated array correctly.
   25l Try adding additional array.
   25l Doesn't work. The two displays are almost the
   same. Both have lots of glitches.
   25m Better but a lot of noise on the last display.
   25n Much worse.
   25o Abandon 25n.
   26a Start moving MAX7219 specific code to a
   child object.
   26a Appears to work correctly.
   Object name is now "Max7219MultiArray131226b".
   26b Appears to work correctly using the parent
   object "Max7219Demo131226b".
   26c Start adding a few features.
   140116b Scrolls sideways but the screens are in
   the wrong order.
   16b Try scrolling other direction.
   16d Comment out debug statements.
   26e Try to fix the way characters are added
   to scroll.
   21a Try to plot pixels with x and y coordinates.
   27c Seems to work well. The delay times have been
   determined to the nearest microsecond.
   27c I still need to move new pixel setting code
   to this object.
   0314a Start adding support of additional rows of
   arrays.
   
}
CON ' Child Constants

  'CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq ' trick used in "Clock.spin" object.
  'ONE_MS = CLK_FREQ / 1000

  ' minimun scroll delay horizontal on four arrays = 25ms 24343us  increased 5891
  ' minimun scroll delay horizontal on three arrays = 19ms 18452us  increased 5891 
  ' minimun scroll delay horizontal on twp arrays = 13ms 12561us   increased 5891
  ' minimun scroll delay horizontal on one array = 7ms 6670us  
  ' minimun scroll delay vertical on four arrays = 23ms 22103us  increased 5330
  ' minimun scroll delay vertical on three arrays = 17ms 16773us increased 5330
  ' minimun scroll delay vertical on twp arrays = 12ms 11443us  increased 5329
  ' minimun scroll delay vertical on one array = 7ms 6114us  
  ' The code as been changed since the above tests. It's possible the
  ' delay amounts are no longer accurate.
  
  DEFAULT_SCROLL_INTERVAL = 100_000 '' User changeable, 100_000 is one tenth of a second.
  LINES_BETWEEN_CHARACTERS = 3  '' User changeable
  SPACE = 32
  FIRST_CHARACTER = SPACE '' First character used in fonts.
  LAST_CHARACTER = 126    '' Last character used in fonts. 
  LAST_CHARACTER_INDEX = LAST_CHARACTER - FIRST_CHARACTER
  CHARACTERS = LAST_CHARACTER_INDEX + 1

  MIN_OVERHEAD_HORIZONTAL_US = 779 ' Delay required to add new column.
  MIN_OVERHEAD_VERTICAL_US = 784
  MIN_DELAY_HORIZONTAL_US = 5_891 ' Additional delay required for each array.
  MIN_DELAY_VERTICAL_US = 5_330
 
  MAX_DELAY_US = 10_000_000
  
CON
'' MAX7219 register names

  ROW_0 = 1
  ROW_1 = 2
  ROW_2 = 3
  ROW_3 = 4
  ROW_4 = 5
  ROW_5 = 6            
  ROW_6 = 7         
  ROW_7 = 8             
   
  DECODE_MODE = 9               ' set to zero for 8x8 array                     
  BRIGHTNESS = 10               ' brightness register 0 - 31
  SCAN_LIMIT = 11               ' limit number or rows scanned set to zero                                   
  SHUT_DOWN = 12                ' set to zero to turn off display
  DISPLAY_TEST = 15             ' set to non-zero to turn on all LEDs

  MIN_BRIGHTNESS = 0
  MAX_BRIGHTNESS = 15 '31
  HALF_BRIGHTNESS = (MAX_BRIGHTNESS + 1) / 2
  QUARTER_BRIGHTNESS = (MAX_BRIGHTNESS + 2) / 4
  EIGHTH_BRIGHTNESS = (MAX_BRIGHTNESS + 4) / 8
  DEFAULT_BRIGHTNESS = 2 'QUARTER_BRIGHTNESS '' User changeable

  LINES_IN_ARRAY = 8
  COLUMNS_IN_ARRAY = LINES_IN_ARRAY
     
  'activeFontId
  #0, FREE_DESIGN, ASCII_0508

  MAX_FONT_INDEX = ASCII_0508
  FONTS_IN_USE = MAX_FONT_INDEX + 1

CON '' Eye Constants

  ' activeEye enumeration
  #0, EYE_NORMAL, EYE_SAD, EYE_SURPRISE, EYE_HAPPY, EYE_FEAR, EYE_CROSSED
      EYE_OVAL_VERT, EYE_OVAL_HOR, EYE_SLIT_VERT0, EYE_SLIT_VERT1
      EYE_SLIT_HOR0, EYE_SLIT_HOR1
      EYE_NORMAL_BLINK_0, EYE_NORMAL_BLINK_1, EYE_NORMAL_BLINK_2
      EYE_NORMAL_BLINK_3, EYE_NORMAL_BLINK_4, EYE_NORMAL_BLINK_5
      EYE_NORMAL_BLINK_6
      EYE_SAD_BLINK_0, EYE_SAD_BLINK_1, EYE_SAD_BLINK_2
      EYE_SAD_BLINK_3
      EYE_SURPRISE_BLINK_0, EYE_SURPRISE_BLINK_1, EYE_SURPRISE_BLINK_2
      EYE_SURPRISE_BLINK_3
      EYE_SLIT_VERT_BLINK_0, EYE_SLIT_VERT_BLINK_1, EYE_SLIT_VERT_BLINK_2
      EYE_SLIT_VERT_BLINK_3

  MAX_EYE_INDEX = EYE_SURPRISE_BLINK_3
  EYE_TYPES = MAX_EYE_INDEX + 1
  NORMAL_BLINKS = 7
  SAD_BLINKS = 4
  SURPRISE_BLINKS = 4
  SLIT_VERT_BLINKS = 4
  TYPES_OF_BLINKING_EYES = 4
  'EYE_HEIGHT = 8
  'EYE_WIDTH = 8
  'EYES_WIDTH = EYE_WIDTH * 2
  'EYES_WIDTH_BYTES = EYES_WIDTH * TRI_COLOR 
  'EYE_PIXELS = EYE_HEIGHT * EYE_WIDTH
  'EYE_BYTES = EYE_PIXELS * TRI_COLOR
  'EYES_PIXELS = EYE_PIXELS * 2
  'EYES_BYTES = EYE_BYTES * 2
                         
VAR

  long arraysInUse, bytesToMove', linesInDisplay
  long arrayAddress
  long scrollTimer, scrollInterval
  long fontAddress[FONTS_IN_USE]
  long maxArrayIndex, oneMs, oneUs
  long maxX, maxY, columnsInUse, rowsInUse
  long resyncInterval, bytePerRowOfArrays

  byte characterWidth[FONTS_IN_USE * CHARACTERS]
  byte activeFontId
  byte clockPin, dataPin, csPin, rotatedFlag
  byte minimumDelay
  byte eyeWidthMargin[EYE_TYPES], eyeHeightMargin[EYE_TYPES]
  
PUB Init(clockPin_, dataPin_, csPin_, columnsInUse_, rowsInUse_, arrayAddress_) | row, column, fontPtr

  clockPin := clockPin_
  dataPin := dataPin_
  csPin := csPin_
  SetNumberOfArrays(columnsInUse_, rowsInUse_)
  SetArrayAddress(arrayAddress_)
  'SetRotation(rotatedFlag_)
  
  oneMs := clkfreq / 1000
  oneUs := clkfreq / 1_000_000

  SetScrollDelay(DEFAULT_SCROLL_INTERVAL)
  activeFontId := FREE_DESIGN
  fontAddress[FREE_DESIGN] := @freeDesign032
  'fontAddress[ASCII_0508] := @ASCII_0508_032

  dira[csPin] := 1
  dira[dataPin] := 1
  dira[clockPin] := 1
  outa[csPin] := 1             '' active low                                    

  '' Note: Element zero of the "defaultRegisters" is not used.
  'repeat result from ROW_0 to SHUT_DOWN ' set defaults for 8x8 array
  '  ShiftOut(result, defaultRegisters[result])
  'Reset
  repeat result from ROW_0 to SHUT_DOWN ' set defaults for 8x8 array
    repeat 2
      ShiftOut(result, defaultRegisters[result])
 
  repeat arraysInUse
    'ShiftOut(DISPLAY_TEST, 1) ' this line is not required
    'waitcnt(clkfreq / 8 + cnt)
    ShiftOut(DISPLAY_TEST, 0) ' this line is not required
    waitcnt(clkfreq / 2 + cnt)
  ShiftOut(DISPLAY_TEST, 0)    
  'Reset
  
  
  repeat maxArrayIndex  ' cycle last writes through all arrays.
    ShiftOut(0, 0)

  InitEyes
  
  '' Find the width of each character and store the value
  '' in the "characterWidth" array. This allows text
  '' which is scrolled to be appropriately spaced.
  '' It may be a good idea to store these values to EEPROM
  '' so they don't have to be calculated each time
  '' the program starts.
           
  repeat result from 0 to MAX_FONT_INDEX  ' find width of both fonts       
    FindWidth(@characterWidth + (result * CHARACTERS), fontAddress[result], CHARACTERS)  
    fontPtr := fontAddress[result]

  '' Eyes not currently used. 
  'FindWidth(@eyeWidth, @eyeNormal, EYE_TYPES)
  'fontPtr := @eyeNormal
  'FindHeight(@eyeHeight, @eyeNormal, EYE_TYPES) 
      
PRI InitEyes | eyePtr

  eyePtr := @eyeNormal
  repeat EYE_TYPES
    eyeHeightMargin[result] := 8 - FindBitmapHeight(eyePtr)
    eyeWidthMargin[result++] := 8 - FindBitmapWidth(eyePtr)
    eyePtr += 8

PUB Reset

  repeat result from ROW_0 to SHUT_DOWN ' set defaults for 8x8 array
    'ShiftOut(result, defaultRegisters[result])
    ShiftOutMultiple(result, defaultRegisters[result], arraysInUse)
    
{PUB SetRotation(newRotation)

  rotatedFlag := newRotation
  result := SetMinimumDelay
}  
PUB SetArrayAddress(newAddress)

  arrayAddress := newAddress
  
PUB SetNumberOfArrays(columnsInUse_, rowsInUse_)

  columnsInUse := columnsInUse_
  rowsInUse := rowsInUse_
  arraysInUse := 1 #> columnsInUse_ * rowsInUse_
  bytePerRowOfArrays := columnsInUse * 8
  SetMinimumDelay
  maxX := (columnsInUse * 8) - 1
  maxY := (rowsInUse * 8) - 1
  'linesInDisplay := arraysInUse * LINES_IN_ARRAY
  
  bytesToMove := bytePerRowOfArrays - 1
  maxArrayIndex := arraysInUse - 1
  
PRI SetMinimumDelay

  if rotatedFlag
    minimumDelay := (arraysInUse * MIN_DELAY_VERTICAL_US) + MIN_OVERHEAD_VERTICAL_US
  else
    minimumDelay := (arraysInUse * MIN_DELAY_HORIZONTAL_US) + MIN_OVERHEAD_HORIZONTAL_US
  result := SetScrollDelay(scrollInterval / oneUs)
  
PUB SetFont(newFont)

  result := activeFontId := 0 #> newFont <# MAX_FONT_INDEX
  
PUB SetBrightness(newBrightness)

  newBrightness := MIN_BRIGHTNESS #> newBrightness <# MAX_BRIGHTNESS
  ShiftOut(BRIGHTNESS, newBrightness)
  repeat maxArrayIndex
    ShiftOut(0, 0)
  result := newBrightness
    
PUB SetScrollDelay(newDelay)

  newDelay := minimumDelay #> newDelay <# MAX_DELAY_US
  scrollInterval := newDelay * oneUs
  'scrollInterval := MIN_SCROLL_INTERVAL #> scrollInterval
  resyncInterval := (scrollInterval * 3) / 2
  result := newDelay
  
PUB UpdateBufferPixel(localX, localY, localState) | arrayIndex, localRegister
'' This method uses "SetBufferPixel" to set the pixel in the buffer
'' and also updated the display by writing the appropriate register
'' of the MAX7219 chip.
 
  result := SetBufferPixel(localX, localY, localState)
  if result == -1
    return
  arrayIndex := localX / 8
  localRegister := (result // 8) + ROW_0
  ShiftOutSingle(arrayIndex, localRegister, byte[arrayAddress][result])
  
PUB SetBufferPixel(localX, localY, localState) | columnIndex, rowIndex, arrayIndex, localBit
'' Changes bit value at location localX, localY in active array to localState.
'' Returns the index value of the byte containing the pixel.
'' Returns -1 if the coordinates are outside the buffer area.
'' The changes made to the buffer do not change the LEDs until
'' "LoadBitsToArray" is called or the register with the
'' updated pixel is written.

  if localX < 0 or localY < 0 or localX > maxX or localY > maxY
    return -1
  arrayIndex := localX / 8
  columnIndex := arrayIndex // columnsInUse
  rowIndex := arrayIndex / columnsInUse
  localY //= 8
  if rotatedFlag == 1
    localBit := 1 << localY '(7 - localY)
    result := localX 
    if localState
      byte[arrayAddress][result] |= localBit
    else
      byte[arrayAddress][result] &= !localBit
  else
    localBit := 1 << (7 - (localX // 8))
    result := (arrayIndex * 8) + localY
    if localState
      byte[arrayAddress][result] |= localBit
    else
      byte[arrayAddress][result] &= !localBit

PUB PlotToBuffer(localX, localY, oldX, oldY) | arrayIndex, bufferIndex[2], localRegister
'' This method light a LED at location localX, localY and it will set the
'' LED at oldX, oldY off.
'' Either coordinates may be left out by setting them to -1.
'' The change of LED state is also written to the active array buffer.

  if localX == oldX and localY == oldY
    return
  bufferIndex[0] := SetBufferPixel(oldX, oldY, 0) 
  bufferIndex[1] := SetBufferPixel(localX, localY, 1)
 
  if bufferIndex[0] <> -1
    arrayIndex := bufferIndex[0] / 8
    localRegister := (bufferIndex[0] // 8) + ROW_0
    ShiftOutSingle(arrayIndex, localRegister, byte[arrayAddress][bufferIndex[0]])
   
  if bufferIndex[0] <> bufferIndex[1] ' if the old and new pixels are not in the same byte (register)
    arrayIndex := bufferIndex[1] / 8 'localX / 8
    localRegister := (bufferIndex[1] // 8) + ROW_0
    ShiftOutSingle(arrayIndex, localRegister, byte[arrayAddress][bufferIndex[1]])

PUB ScrollText(messagePtr)

  SyncScrollTime
  repeat
    ScrollCharacter(messagePtr++)  
  while byte[messagePtr]

{PUB ScrollRomText(messagePtr)

  SyncScrollTime
  repeat
    ScrollRomCharacter(messagePtr++)  
  while byte[messagePtr]
  
PRI ScrollRomCharacter(characterPtr) | fontPtr, localSize   
'' Font does not need to be rotated.

  MoveFontFromRomToRam(byte[characterPtr], @scratchRomCharacter, 2)

  ScrollBitmapTall(@scratchRomCharacter, 16, 4, 2)
  
  ScrollBlank(LINES_BETWEEN_CHARACTERS)

PUB ScrollBitmapTall(localPtr, pixelColumnsToScroll, arrayHeight, memoryWidth) | colIndex, rowIndex, bytePtr[4]

  colIndex := 0
  rowIndex := 0
  bytePtr[0] := localPtr
  bytePtr[1] := localPtr + (memoryWidth * 8)
  bytePtr[2] := localPtr + (memoryWidth * 2 * 8)
  bytePtr[3] := localPtr + (memoryWidth * 3 * 8)
  if pixelColumnsToScroll
    pixelColumnsToScroll--
    repeat result from 0 to pixelColumnsToScroll
      if result // 8 == 0
        repeat rowIndex from 0 to arrayHeight - 1
          bytePtr[rowIndex]++ 
      repeat rowIndex from 0 to arrayHeight - 1
        AddLineTall(GetColumnOfBitsTall(localPtr[rowIndex], result, arrayHeight, memoryWidth), rowIndex)
  
PUB GetColumnOfBitsTall(bitmapPtr, fromLeftEdge, arrayHeight, memoryWidth)
'' MSB from bottom of bitmap.

  repeat LINES_IN_ARRAY
    result <<= 1
    result += (byte[bitmapPtr++] >> (7 - fromLeftEdge)) & 1
    
PUB AddLineTall(newLine, rowIndex) : localPtr    '************************************
'' The bitmap is stored differently for rotated arrays.

  localPtr := arrayAddress + (rowIndex * bytePerRowOfArrays)
  if rotatedFlag
    newLine ><= 8 ' Turn right side up.
    bytemove(localPtr, localPtr + 1, bytesToMove)
    byte[localPtr][bytesToMove] := newLine  
 { else  
    localPtr := arrayAddress + (maxArrayIndex * LINES_IN_ARRAY) 'Top row of last array     
    repeat arraysInUse
      newLine := ShiftColumn(localPtr, newLine)
      localPtr -= LINES_IN_ARRAY  }

  LoadBitsToArray(arrayAddress)
  NextScrollTime

PUB ShiftColumnTall(localPtr, leastSignificantBits) : mostSignificantBits 
'' MSB from bottom of bitmap.

  leastSignificantBits ->= 8
  
  repeat LINES_IN_ARRAY
    mostSignificantBits <<= 1
    mostSignificantBits |= (byte[localPtr] & %1000_0000) >> 7
    byte[localPtr] <<= 1
    leastSignificantBits <-= 1
    byte[localPtr] |= leastSignificantBits & 1
    localPtr++
 }
PRI ScrollCharacter(characterPtr) | fontPtr, localSize   
'' Font does not need to be rotated.

  fontPtr := fontAddress[activeFontId] + (LINES_IN_ARRAY * (byte[characterPtr] - FIRST_CHARACTER))
  localSize := characterWidth[(activeFontId * CHARACTERS) + (byte[characterPtr] - FIRST_CHARACTER)]

  ScrollBitmap(fontPtr, localSize)
  
  ScrollBlank(LINES_BETWEEN_CHARACTERS)

PUB ScrollBitmap(localPtr, columnsToScroll) | fontPtr, localSize   

  if columnsToScroll
    columnsToScroll--
    repeat result from 0 to columnsToScroll
      AddLine(GetColumnOfBits(localPtr, result))
  
PUB ScrollBlank(newLines)

  repeat newLines
    AddLine(0)
  
PUB AddLine(newLine) : localPtr 
'' The bitmap is stored differently for rotated arrays.

  if rotatedFlag
    newLine ><= 8 ' Turn right side up.
    bytemove(arrayAddress, arrayAddress + 1, bytesToMove)
    byte[arrayAddress][bytesToMove] := newLine  
  else  
    localPtr := arrayAddress + (maxArrayIndex * LINES_IN_ARRAY) 'Top row of last array     
    repeat arraysInUse
      newLine := ShiftColumn(localPtr, newLine)
      localPtr -= LINES_IN_ARRAY

  LoadBitsToArray(arrayAddress)
  NextScrollTime
  
PUB GetColumnOfBits(bitmapPtr, fromLeftEdge)
'' MSB from bottom of bitmap.

  repeat LINES_IN_ARRAY
    result <<= 1
    result += (byte[bitmapPtr++] >> (7 - fromLeftEdge)) & 1
    
PUB ShiftColumn(localPtr, leastSignificantBits) : mostSignificantBits 
'' MSB from bottom of bitmap.

  leastSignificantBits ->= 8
  
  repeat LINES_IN_ARRAY
    mostSignificantBits <<= 1
    mostSignificantBits |= (byte[localPtr] & %1000_0000) >> 7
    byte[localPtr] <<= 1
    leastSignificantBits <-= 1
    byte[localPtr] |= leastSignificantBits & 1
    localPtr++

PUB DisplayEye(eyeIndex, fromSide, fromTop) | eyePtr, localPtr, temp

  fromSide := 0 #> (eyeWidthMargin[eyeIndex] - 1)/ 2 + fromSide <# eyeWidthMargin[eyeIndex]
  fromTop := 0 #> (eyeHeightMargin[eyeIndex] - 1) / 2 + fromTop <# eyeHeightMargin[eyeIndex]

  eyePtr := @eyeNormal + eyeIndex * LINES_IN_ARRAY
  localPtr := arrayAddress '@localBuffer
  {bytefill(localPtr, 0, 16) 'fromTop)
  bytefill(localPtr, 255, fromSide) 'fromTop)
  LoadBitsToArray(arrayAddress)
  'return
  waitcnt(clkfreq / 2 + cnt)}
  
  if fromTop
    bytefill(localPtr, 0, fromTop)
    localPtr += fromTop
  result := 8 - fromTop
  repeat result 
    byte[localPtr++] := byte[eyePtr++] >> fromSide

  eyePtr := @eyeNormal + eyeIndex * LINES_IN_ARRAY
    
  if fromTop
    bytefill(localPtr, 0, fromTop)
    localPtr += fromTop
  result := 8 - fromTop
  repeat result
    temp := byte[eyePtr++] >< 8
    byte[localPtr++] := temp << fromSide

  {bytemove(arrayAddress, @eyeNormal + eyeIndex * LINES_IN_ARRAY, LINES_IN_ARRAY)

    
  longfill(@localBuffer, 0, 2)
  bytemove(@localBuffer + fromTop, @eyeNormal + eyeIndex * LINES_IN_ARRAY, LINES_IN_ARRAY) 
  
  bytemove(arrayAddress, @eyeNormal + eyeIndex * LINES_IN_ARRAY, LINES_IN_ARRAY)
  bytemove(arrayAddress + LINES_IN_ARRAY, @eyeNormal + eyeIndex * LINES_IN_ARRAY, LINES_IN_ARRAY) }
  LoadBitsToArray(arrayAddress)

{PUB ShiftColumnOtherWay(localPtr, mostSignificantBits) : leastSignificantBits
'' Not presently used.

  mostSignificantBits ->= 8
  
  repeat LINES_IN_ARRAY
    leastSignificantBits <<= 1
    'leastSignificantBits |= (byte[localPtr] & %1000_0000) >> 7
    leastSignificantBits |= byte[localPtr] & %1
    byte[localPtr] >>= 1
    mostSignificantBits <-= 1
    byte[localPtr] |= (mostSignificantBits & 1) << 7
    localPtr++
            }
PUB ClearArray

  ClearBuffer(arrayAddress)

PUB ClearBuffer(localPtr)

  bytefill(localPtr, 0, arraysInUse * LINES_IN_ARRAY)

PUB SyncScrollTime

  scrollTimer := cnt '+ scrollInterval

PUB NextScrollTime
'' This shouldn't hang if the interval has expired.
  
  repeat
    result := cnt - scrollTimer' - cnt
    if result < 0
      result := resyncInterval
  while result < scrollInterval
  if result > resyncInterval
    SyncScrollTime
  else
    scrollTimer += scrollInterval

{PUB DisplayRotatedEye(eyeIndex) | lastEyeIndex, lineIndex
'' Not presently used.

  bytefill(arrayAddress, 0, arraysInUse * LINES_IN_ARRAY)
  if arraysInUse => 2
    lastEyeIndex := (2 * LINES_IN_ARRAY) - 1
  if eyeAlignFromOuter[eyeIndex]    
    repeat lineIndex from 0 to eyeAlignFromOuter[eyeIndex] - 1' fill empty space around outside of eyes
      byte[arrayAddress][lineIndex] := 0
      if arraysInUse => 2
        byte[arrayAddress][lastEyeIndex - lineIndex] := 0
  lineIndex := eyeAlignFromOuter[eyeIndex]
  repeat eyeWidth
    byte[arrayAddress][lineIndex] := byte[@eyeNormal][(eyeIndex * LINES_IN_ARRAY)] << eyeAlignFromTop[eyeIndex]
    if arraysInUse => 2
      byte[arrayAddress][lastEyeIndex - lineIndex] := byte[arrayAddress][lineIndex]
  LoadBitsToArray(arrayAddress)
}       
PRI FindWidth(destination, source, localSize) '| characterIndex, bitIndex
'' This method should be called before rotating the font.
'' The font being measured should 8x8
'' The font should be left justified.

  result := destination
  repeat localSize
    byte[destination++] := FindBitmapWidth(source)  
    source += LINES_IN_ARRAY

  byte[result + (SPACE - FIRST_CHARACTER)] := byte[result + ("n" - FIRST_CHARACTER)] - LINES_BETWEEN_CHARACTERS
  
PRI FindBitmapWidth(localPtr) | rowIndex, bitIndex
'' Assumes the bitmap is left justified.

  repeat rowIndex from 0 to 7
    outb[7..0] := byte[localPtr][rowIndex]
    repeat bitIndex from 0 to 7
      if outb[7 - bitIndex]
        if bitIndex + 1 > result
          result := bitIndex + 1 
  
PRI FindAndStoreBitmapHeights(destination, source, localSize)
'' The bitmap being measured should 8x8
'' The height is will be 8 - however many empty
'' bytes are at the bottom of the bitmap.

  repeat localSize
    byte[destination++] := FindBitmapHeight(source)
    source += LINES_IN_ARRAY
    
PUB FindBitmapHeight(localPtr) | tempPtr

  tempPtr := localPtr + 7
  result := 8
  repeat while byte[tempPtr] == 0 and tempPtr => localPtr
    result--
    tempPtr--

PUB LoadBitsToArray(localPtr) : row | arrayIndex  
                          
  repeat row from ROW_0 to ROW_7  
    outa[csPin] := 0
    repeat arrayIndex from maxArrayIndex to 0 
    
      ClockOut(row, byte[localPtr + (arrayIndex * LINES_IN_ARRAY)])
    outa[csPin] := 1  
    localPtr++   
 
PUB ShiftOutMultiple(address, data, arrays)

  outa[csPin] := 0
    repeat arrays
      ClockOut(address, data)
  outa[csPin] := 1

PUB ShiftOut(address, data)

  outa[csPin] := 0
  ClockOut(address, data)
  outa[csPin] := 1

PUB ShiftOutSingle(arrayIndex, address, data)
'' When using multiple arrays, data which
'' is to be sent to a single array will need
'' to have blank data sent before and/or
'' behind it. For example if there were
'' three arrays in use and the brightness of
'' the center arrays was to be set to zero
'' while leaving the two other arrays at
'' the original brightness level, then the
'' call of ShiftOutSingle(1, BRIGHTNESS, 0)
'' would cause blank data to be sent both
'' before and after the brightness setting
'' command.

  outa[csPin] := 0
  result := (arraysInUse - 1) - arrayIndex
  if result > 0
    repeat result
      ClockOut(0, 0)
  ClockOut(address, data)
  if arrayIndex
    repeat arrayIndex
      ClockOut(0, 0)
  outa[csPin] := 1

{PUB SetAsOutput(pinPtr, numberOfOuputs)
'' Not currently used.

  if numberOfOuputs < 1
    return
  numberOfOuputs--  
  repeat result from 0 to numberOfOuputs
    dira[byte[pinPtr][result]] := 1

PUB ReleaseOutputs(pinPtr, numberOfOuputs)
'' Not currently used.

  if numberOfOuputs < 1
    return
  numberOfOuputs--  
  repeat result from 0 to numberOfOuputs
    dira[byte[pinPtr][result]] := 0
   
PUB UpdatedParallelArrays(dataPtr, dataPinPtr, scratchLongPtr, numberOfDataLines, arraysPerDataLine) | tempPtr
'' Not currently used.
    
  repeat result from ROW_0 to ROW_7
    tempPtr := dataPtr
    outa[csPin] := 0
    repeat arraysPerDataLine
      ClockOutDatas(result, tempPtr, 8 * arraysPerDataLine, dataPinPtr, scratchLongPtr, numberOfDataLines)
      tempPtr += 8
    dataPtr += 1
    outa[csPin] := 1
    
PUB ClockOutDatas(address, dataPtr, dataOffset, dataPinPtr, scratchLongPtr, numberOfDataLines) | maxLineIndex
'' "address" is the register to be written.
'' "dataPtr" is the address of an array of bytes containing the data to be written
'' to the MAX7219 chips. 
'' "dataOffset" indicates how data to be clocked is distributed within the array
'' located at "dataPtr".
'' "dataOffset" will usually have the value  of 8 or 1. This could be 16 or other multiple
'' of 8 is each data line controls multiple arrays.
'' "dataPinPtr" is the address of an array of bytes containing the I/O pins connected
'' to the data lines.
'' "scratchLongPtr" is the address of an array of longs used to contain the address
'' and data info while the data to make it easier to clock out the combined data.
'' "numberOfDataLines" is the number of independent data lines used.
'' The I/O pins used as data lines should be set as outputs prior to calling this method.
'' Not currently used.

  maxLineIndex := numberOfDataLines - 1
  repeat result from 0 to maxLineIndex
    long[scratchLongPtr][result] := (address << 8) + byte[dataPtr][result * dataOffset]
    long[scratchLongPtr][result] <<= 16
  repeat 16
    repeat result from 0 to maxLineIndex
      long[scratchLongPtr][result] <-= 1
      outa[byte[dataPinPtr][result]] := long[scratchLongPtr][result]
      outa[byte[dataPinPtr][result]] := 0
 }
PUB ClockOut(address, data)

  result := (address << 8) + data
  result <<= 16
  repeat 16
    result <-= 1
    outa[dataPin] := result
    outa[clockPin] := 1
    outa[clockPin] := 0
   
{PUB ClockOutReverse(address, data)
'' Not currently used.

  data ><= 8                    ' reverse the data
  result := (address << 8) + data
  result <<= 16
  repeat 16
    result <-= 1
    outa[dataPin] := result
    outa[clockPin] := 1
    outa[clockPin] := 0
 }
PUB GetObjectName

  result := @objectName

{PUB IdentifyDisplays | localCharacter

  result := arrayAddress
  localCharacter := "1"
  repeat arraysInUse
    if rotatedFlag
      RotateCharacter(activeFontId, localCharacter, result)
    else
      bytemove(result, fontAddress[activeFontId] + (8 * (localCharacter - FIRST_CHARACTER)), 8)
    result += 8
    localCharacter++
  LoadBitsToArray(arrayAddress)
         
PUB RotateCharacter(localFont, localCharacter, destination)

  result := fontAddress[localFont] + (8 * (localCharacter - FIRST_CHARACTER))
  if destination == -1
    destination := result
  RotateBitmap8x8(destination, result)
  
PUB RotateBitmap8x8(destination, source) | temp[2], scratch[2], rowOld, rowNew, colOld, colNew, testResult, testBit, bitOfInterest
'' This method rotates the bits in eight bytes
'' located at "source" and places the rotated bytes
'' in "destination".
'' "source" and "destination" may be the same
'' location.
'' The rotation is 90 degrees clockwise

  'Pst.Str(string(13, " RotateCharacter"))
    
  longfill(@temp, 0, 2)
  bytemove(@scratch, source, LINES_IN_ARRAY) 
  repeat rowNew from 0 to 7
    bitOfInterest := 7 - rowNew
    repeat rowOld from 0 to 7
      dirb[7..0] := byte[@scratch][rowOld]
      outb[rowOld] := dirb[bitOfInterest]
    byte[@temp][rowNew] := outb
    'Pst.Str(string(13, " %"))
    'Pst.Bin(byte[@temp][rowNew], LINES_IN_ARRAY)
              
  bytemove(destination, @temp, LINES_IN_ARRAY)
  
PUB MoveFontFromRomToRam(character, localPtr, ramWidth) | bytePtr, romPtr, firstOfPixelRow, {
} firstOfArrayRow, pixelPtr, scratchBit, oddFlag, bytePerRowOfBuffer
'' Only use this method when array is 32 pixels high.

  bytePerRowOfBuffer := 8 * ramWidth
  oddFlag := character & 1
 
  bytePtr := firstOfArrayRow := firstOfPixelRow := localPtr
   
  romPtr := $8000 + ((character / 2) * 128)
 
  if character => 32 and character =< 126
 
  repeat 4
    repeat 8
      result := long[romPtr]
      result ><= 32
      result := SquishLongAndSwap(result, oddFlag)
      pixelPtr := @result 
      repeat 2      
        bytemove(bytePtr, pixelPtr++, 1)
        bytePtr += 8
      romPtr += 4
      firstOfPixelRow++
      bytePtr := firstOfPixelRow
    firstOfArrayRow += bytePerRowOfBuffer '32
    bytePtr := firstOfPixelRow := firstOfArrayRow

  if rotatedFlag
    RotateArrays(localPtr, ramWidth * 4)
 }         
PUB SquishLongAndSwap(longToSquish, oddFlag) | sourceBit, destinationBit, sourceToDestinationShiftAmount

  sourceBit := 1
  sourceToDestinationShiftAmount := 0
  ifnot oddFlag
    longToSquish >>= 1
  repeat 16  
    destinationBit := longToSquish & sourceBit
    destinationBit >>= sourceToDestinationShiftAmount
    result |= destinationBit
    sourceBit <<= 2
    sourceToDestinationShiftAmount += 1
 
  result.byte[2] := result.byte[1]
  result.byte[1] := result.byte[0]
  result.byte[0] := result.byte[2]
      
{PUB RotateArrays(localPtr, numberOfArraysToRotate)

  repeat numberOfArraysToRotate
    RotateBitmap8x8(localPtr, localPtr)
    localPtr += 8
}
DAT
'' Element zero of the "defaultRegisters" is not used.                                   
defaultRegisters        byte 0[10], DEFAULT_BRIGHTNESS, 255, 1
                                  
DAT freeDesign032       byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000

freeDesign033           byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %00000000
                        byte %11000000
              
freeDesign034           byte %10100000
                        byte %10100000
                        byte %10100000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                         
freeDesign035           byte %01010000
                        byte %11111000
                        byte %01010000
                        byte %11111000
                        byte %01010000
                        byte %00000000
                        byte %00000000
                        byte %00000000

freeDesign036           byte %01000000
                        byte %01110000
                        byte %11000000
                        byte %11100000
                        byte %01110000
                        byte %00110000
                        byte %11100000
                        byte %01000000
              
freeDesign037           byte %01111100
                        byte %10100100
                        byte %10100100
                        byte %01001000
                        byte %00010010
                        byte %00100101
                        byte %00100101
                        byte %00100010
              
freeDesign038           byte %00111100
                        byte %01100100
                        byte %01100000
                        byte %00110000
                        byte %01000110
                        byte %11001100
                        byte %11001100
                        byte %01110110
              
freeDesign039           byte %10000000
                        byte %10000000
                        byte %10000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000

freeDesign040           byte %00100000
                        byte %01000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %01000000
                        byte %00100000

freeDesign041           byte %10000000
                        byte %01000000
                        byte %01100000
                        byte %01100000
                        byte %01100000
                        byte %01100000
                        byte %01000000
                        byte %10000000
                         
freeDesign042           byte %00000000
                        byte %00000000
                        byte %00100000
                        byte %10101000
                        byte %01110000
                        byte %10101000
                        byte %00100000
                        byte %00000000

freeDesign043           byte %00000000
                        byte %00000000
                        byte %00100000
                        byte %00100000
                        byte %11111000
                        byte %00100000
                        byte %00100000
                        byte %00000000

freeDesign044           byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %11000000
                        byte %01000000
                        byte %10000000
              
freeDesign045           byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %11100000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
              
freeDesign046           byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %11000000
                        byte %11000000
              
freeDesign047           byte %00010000
                        byte %00010000
                        byte %00100000
                        byte %00100000
                        byte %01000000
                        byte %01000000
                        byte %10000000
                        byte %10000000
              
freeDesign048           byte %01111000
                        byte %11001100
                        byte %11001100
                        byte %11101100
                        byte %11011100
                        byte %11001100
                        byte %11001100
                        byte %01111000
              
freeDesign049           byte %01100000
                        byte %11100000
                        byte %01100000
                        byte %01100000
                        byte %01100000
                        byte %01100000
                        byte %01100000
                        byte %01100000
              
freeDesign050           byte %01110000
                        byte %10011000
                        byte %00011000
                        byte %00011000
                        byte %00110000
                        byte %01100000
                        byte %11000000
                        byte %11111000
{ Example of rotated characters
                        byte %00000010
                        byte %11111111
                        byte %11111111
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
              
                        byte %11000010
                        byte %11100001
                        byte %10110001
                        byte %10011111
                        byte %10001110
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        
}              
freeDesign051           byte %01110000
                        byte %10011000
                        byte %00011000
                        byte %01110000
                        byte %00011000
                        byte %00011000
                        byte %00011000
                        byte %11110000
              
freeDesign052           byte %00011000
                        byte %00011000
                        byte %01011000
                        byte %01011000
                        byte %10011000
                        byte %11111100
                        byte %00011000
                        byte %00011000
              
freeDesign053           byte %11111000
                        byte %11000000
                        byte %11000000
                        byte %11110000
                        byte %00011000
                        byte %00011000
                        byte %00011000
                        byte %11110000
              
freeDesign054           byte %00111000
                        byte %01000000
                        byte %11000000
                        byte %11111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %01111000
              
freeDesign055           byte %11111100
                        byte %00001100
                        byte %00011000
                        byte %00011000
                        byte %00110000
                        byte %00110000
                        byte %01100000
                        byte %01100000
              
freeDesign056           byte %01111000
                        byte %11001100
                        byte %11001100
                        byte %01111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %01111000
              
freeDesign057           byte %01111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %01111100
                        byte %00001100
                        byte %00011000
                        byte %01110000
              
freeDesign058           byte %00000000
                        byte %00000000
                        byte %11000000
                        byte %11000000
                        byte %00000000
                        byte %11000000
                        byte %11000000
                        byte %00000000

freeDesign059           byte %00000000
                        byte %00000000
                        byte %11000000
                        byte %11000000
                        byte %00000000
                        byte %11000000
                        byte %01000000
                        byte %10000000
                         
freeDesign060           byte %00000000
                        byte %00110000
                        byte %01100000
                        byte %11000000
                        byte %11000000
                        byte %01100000
                        byte %00110000
                        byte %00000000
              
freeDesign061           byte %00000000
                        byte %00000000
                        byte %11100000
                        byte %00000000
                        byte %11100000
                        byte %00000000
                        byte %00000000
                        byte %00000000
              
freeDesign062           byte %00000000
                        byte %11000000
                        byte %01100000
                        byte %00110000
                        byte %00110000
                        byte %01100000
                        byte %11000000
                        byte %00000000
              
freeDesign063           byte %11111000
                        byte %11001100
                        byte %00001100
                        byte %00011000
                        byte %00110000
                        byte %00100000
                        byte %00000000
                        byte %00110000
              
freeDesign064           byte %00111110
                        byte %01000001
                        byte %10011101
                        byte %10100101
                        byte %10100101
                        byte %10111111
                        byte %01000000
                        byte %00111110
              
freeDesign065           byte %00110000
                        byte %00110000
                        byte %00111000
                        byte %01011000
                        byte %01011000
                        byte %11111100
                        byte %10001100
                        byte %10001100
              
freeDesign066           byte %11111000
                        byte %11001100
                        byte %11001100
                        byte %11111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11111000
              
freeDesign067           byte %01111000
                        byte %11001000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11001000
                        byte %01110000
              
freeDesign068           byte %11111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11111000
              
freeDesign069           byte %11111000
                        byte %11000000
                        byte %11000000
                        byte %11110000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11111000
                         
freeDesign070           byte %11111000
                        byte %11000000
                        byte %11000000
                        byte %11110000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
              
freeDesign071           byte %01111100
                        byte %11000100
                        byte %11000000
                        byte %11000000
                        byte %11011100
                        byte %11001100
                        byte %11001100
                        byte %01110100
                         
freeDesign072           byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11111100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
              
freeDesign073           byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
              
freeDesign074           byte %00011000
                        byte %00011000
                        byte %00011000
                        byte %00011000
                        byte %00011000
                        byte %00011000
                        byte %10011000
                        byte %01110000
              
freeDesign075           byte %11000100
                        byte %11001000
                        byte %11010000
                        byte %11110000
                        byte %11011000
                        byte %11011000
                        byte %11001100
                        byte %11001100
              
freeDesign076           byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11111000
                         
freeDesign077           byte %11000110
                        byte %11000110
                        byte %11101110
                        byte %11101110
                        byte %10110110
                        byte %10110110
                        byte %10000110
                        byte %10000110
              
freeDesign078           byte %11001000 
                        byte %11001000
                        byte %11101000
                        byte %11101000
                        byte %10111000
                        byte %10111000
                        byte %10011000
                        byte %10011000
              
freeDesign079           byte %01111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %01111000
              
freeDesign080           byte %11111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11111000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                         
freeDesign081           byte %01111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %01111000
                        byte %00001110
                         
freeDesign082           byte %11111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
              
freeDesign083           byte %01111000
                        byte %11001000
                        byte %11100000
                        byte %01110000
                        byte %00111000
                        byte %00011000
                        byte %10011000
                        byte %11110000
              
freeDesign084           byte %11111100
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
              
freeDesign085           byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %01111000
              
freeDesign086           byte %11000100
                        byte %11000100
                        byte %01101000
                        byte %01101000
                        byte %01101000
                        byte %00110000
                        byte %00110000
                        byte %00110000
              
freeDesign087           byte %11000001
                        byte %11000001
                        byte %01101001
                        byte %01101101
                        byte %01101101
                        byte %00110011
                        byte %00110011
                        byte %00110011
              
freeDesign088           byte %11001000
                        byte %11001000
                        byte %01110000
                        byte %01100000
                        byte %00110000
                        byte %01110000
                        byte %10011000
                        byte %10011000
              
freeDesign089           byte %11000100
                        byte %11000100
                        byte %01101000
                        byte %01101000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
              
freeDesign090           byte %11111000
                        byte %00011000
                        byte %00110000
                        byte %00110000
                        byte %01100000
                        byte %01100000
                        byte %11000000
                        byte %11111000
              
freeDesign091           byte %11110000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11110000

freeDesign092           byte %00000000
                        byte %11000000
                        byte %01100000
                        byte %01100000
                        byte %00110000
                        byte %00110000
                        byte %00011000
                        byte %00000000

freeDesign093           byte %11110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %11110000
                         
freeDesign094           byte %00000000
                        byte %00000000
                        byte %00100000
                        byte %01010000
                        byte %10001000
                        byte %00000000
                        byte %00000000
                        byte %00000000

freeDesign095           byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %11111111
                         
freeDesign096           byte %00000000
                        byte %00000000
                        byte %11000000
                        byte %01100000
                        byte %00110000
                        byte %00000000
                        byte %00000000
                        byte %00000000

freeDesign097           byte %00000000
                        byte %01111000
                        byte %00001100
                        byte %01111100
                        byte %11001100
                        byte %11001100
                        byte %01111100
                        byte %00000000
                         
freeDesign098           byte %11000000
                        byte %11000000
                        byte %11111000
                        byte %11001100 
                        byte %11001100 
                        byte %11001100
                        byte %11111000
                        byte %00000000

freeDesign099           byte %00000000
                        byte %01111000
                        byte %11001000
                        byte %11000000
                        byte %11000000
                        byte %11001000
                        byte %01110000
                        byte %00000000

freeDesign100           byte %00001100
                        byte %00001100
                        byte %01111100
                        byte %11001100 
                        byte %11001100 
                        byte %11001100 
                        byte %01111100
                        byte %00000000

freeDesign101           byte %00000000
                        byte %01111000
                        byte %11001100
                        byte %11011000
                        byte %11100000
                        byte %11000100
                        byte %01111000
                        byte %00000000
              
freeDesign102           byte %00111000
                        byte %01100000
                        byte %11110000
                        byte %01100000
                        byte %01100000
                        byte %01100000
                        byte %01100000
                        byte %00000000
                         
freeDesign103           byte %00000000
                        byte %11111110
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %01111100
                        byte %00001100
                        byte %01111000
                         
freeDesign104           byte %11000000
                        byte %11000000
                        byte %11011000
                        byte %11101100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %00000000

freeDesign105           byte %11000000 
                        byte %00000000
                        byte %11000000 
                        byte %11000000 
                        byte %11000000 
                        byte %11000000 
                        byte %11000000
                        byte %00000000

freeDesign106           byte %01100000 
                        byte %00000000
                        byte %01100000 
                        byte %01100000 
                        byte %01100000 
                        byte %01100000
                        byte %01100000
                        byte %11000000

freeDesign107           byte %11000000 
                        byte %11000100 
                        byte %11001000 
                        byte %11010000 
                        byte %11110000 
                        byte %11011000 
                        byte %11001100
                        byte %00000000

freeDesign108           byte %11000000 
                        byte %11000000
                        byte %11000000 
                        byte %11000000 
                        byte %11000000 
                        byte %11000000 
                        byte %11000000
                        byte %00000000

freeDesign109           byte %00000000
                        byte %11010010
                        byte %11111111
                        byte %11011011
                        byte %11011011
                        byte %11011011
                        byte %11011011
                        byte %00000000

freeDesign110           byte %00000000
                        byte %11011000
                        byte %11101100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %00000000

freeDesign111           byte %00000000
                        byte %01111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %01111000
                        byte %00000000

freeDesign112           byte %00000000
                        byte %11111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11111000
                        byte %11000000
                        byte %11000000
                         
freeDesign113           byte %00000000
                        byte %01111100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %01111100
                        byte %00001100
                        byte %00001100
              
freeDesign114           byte %00000000
                        byte %11010000
                        byte %11110000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %00000000

freeDesign115           byte %00000000
                        byte %01110000
                        byte %11000000
                        byte %11100000
                        byte %01110000
                        byte %00110000
                        byte %11100000
                        byte %00000000

freeDesign116           byte %01100000
                        byte %01100000
                        byte %11110000
                        byte %01100000
                        byte %01100000
                        byte %01100000
                        byte %01100000
                        byte %01100000
              
freeDesign117           byte %00000000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %01111000
                        byte %00000000
                         
freeDesign118           byte %00000000
                        byte %11001100
                        byte %11001100
                        byte %01101000
                        byte %01101000
                        byte %00110000
                        byte %00110000
                        byte %00000000

freeDesign119           byte %00000000
                        byte %11011011
                        byte %11011011
                        byte %11011011
                        byte %11011011
                        byte %01101100
                        byte %01101100
                        byte %00000000
              
freeDesign120           byte %00000000
                        byte %11001100
                        byte %11101000
                        byte %01110000
                        byte %00111000
                        byte %01011100
                        byte %11001100
                        byte %00000000

freeDesign121           byte %00000000
                        byte %11001100
                        byte %11001100
                        byte %01101000
                        byte %01101000
                        byte %00110000
                        byte %00010000
                        byte %01110000
              
freeDesign122           byte %00000000
                        byte %11110000
                        byte %00110000
                        byte %01100000
                        byte %01100000
                        byte %11000000
                        byte %11110000
                        byte %00000000
              
freeDesign123           byte %00111000
                        byte %01100000
                        byte %01100000
                        byte %11000000
                        byte %11000000
                        byte %01100000
                        byte %01100000
                        byte %00111000
                         
freeDesign124           byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
              
freeDesign125           byte %11100000
                        byte %00110000
                        byte %00110000
                        byte %00011000
                        byte %00011000
                        byte %00110000
                        byte %00110000
                        byte %11100000
              
freeDesign126           byte %00000000
                        byte %00000000
                        byte %01110000
                        byte %10011001
                        byte %00001110
                        byte %00000000
                        byte %00000000
                        byte %00000000

DAT
'--------------------------------- [ ASCII ]--------------------------------------
'----------------------------------[  5x8  ]--------------------------------------
{DAT ASCII_0508_032 byte    $00,$00,$00,$00,$00,$00,$00,$00    ' - -           
ASCII_0508_033 byte        $20,$20,$20,$20,$20,$00,$20,$00    ' -!-           
ASCII_0508_034 byte        $50,$50,$50,$00,$00,$00,$00,$00    ' -"-           
ASCII_0508_035 byte        $50,$50,$F8,$50,$F8,$50,$50,$00    ' -#-           
ASCII_0508_036 byte        $20,$78,$C0,$70,$28,$F0,$20,$00    ' -$-           
ASCII_0508_037 byte        $C0,$C8,$10,$20,$40,$98,$18,$00    ' -%-           
ASCII_0508_038 byte        $40,$A0,$A0,$40,$A8,$90,$68,$00    ' -&-           
ASCII_0508_039 byte        $30,$20,$40,$00,$00,$00,$00,$00    ' -'-           
ASCII_0508_040 byte        $10,$20,$40,$40,$40,$20,$10,$00    ' -(-           
ASCII_0508_041 byte        $40,$20,$10,$10,$10,$20,$40,$00    ' -)-           
ASCII_0508_042 byte        $20,$A8,$70,$20,$70,$A8,$20,$00    ' -*-           
ASCII_0508_043 byte        $20,$20,$20,$F8,$20,$20,$20,$00    ' -+-           
ASCII_0508_044 byte        $00,$00,$00,$00,$60,$40,$80,$00    ' -,-           
ASCII_0508_045 byte        $00,$00,$00,$F8,$00,$00,$00,$00    ' ---           
ASCII_0508_046 byte        $00,$00,$00,$00,$00,$60,$60,$00    ' -.-           
ASCII_0508_047 byte        $00,$08,$10,$20,$40,$80,$00,$00    ' -/-           
ASCII_0508_048 byte        $70,$88,$98,$A8,$C8,$88,$70,$00    ' -0-           
ASCII_0508_049 byte        $20,$60,$20,$20,$20,$20,$70,$00    ' -1-           
ASCII_0508_050 byte        $70,$88,$08,$30,$40,$80,$F8,$00    ' -2-           
ASCII_0508_051 byte        $F8,$08,$10,$30,$08,$88,$70,$00    ' -3-           
ASCII_0508_052 byte        $10,$30,$50,$90,$F8,$10,$10,$00    ' -4-           
ASCII_0508_053 byte        $F8,$80,$F0,$08,$08,$88,$70,$00    ' -5-           
ASCII_0508_054 byte        $38,$40,$80,$F0,$88,$88,$70,$00    ' -6-           
ASCII_0508_055 byte        $F8,$08,$10,$20,$40,$40,$40,$00    ' -7-           
ASCII_0508_056 byte        $70,$88,$88,$70,$88,$88,$70,$00    ' -8-           
ASCII_0508_057 byte        $70,$88,$88,$78,$08,$10,$E0,$00    ' -9-           
ASCII_0508_058 byte        $00,$60,$60,$00,$60,$60,$00,$00    ' -:-           
ASCII_0508_059 byte        $00,$60,$60,$00,$60,$60,$80,$00    ' -;-           
ASCII_0508_060 byte        $10,$20,$40,$80,$40,$20,$10,$00    ' -<-           
ASCII_0508_061 byte        $00,$00,$F8,$00,$F8,$00,$00,$00    ' -=-           
ASCII_0508_062 byte        $40,$20,$10,$08,$10,$20,$40,$00    ' ->-           
ASCII_0508_063 byte        $70,$88,$10,$20,$20,$00,$20,$00    ' -?-           
ASCII_0508_064 byte        $70,$88,$B8,$A8,$B8,$80,$78,$00    ' -@-           
ASCII_0508_065 byte        $20,$50,$88,$88,$F8,$88,$88,$00    ' -A-           
ASCII_0508_066 byte        $F0,$88,$88,$F0,$88,$88,$F0,$00    ' -B-           
ASCII_0508_067 byte        $70,$88,$80,$80,$80,$88,$70,$00    ' -C-           
ASCII_0508_068 byte        $F0,$88,$88,$88,$88,$88,$F0,$00    ' -D-           
ASCII_0508_069 byte        $F8,$80,$80,$F0,$80,$80,$F8,$00    ' -E-           
ASCII_0508_070 byte        $F8,$80,$80,$F0,$80,$80,$80,$00    ' -F-           
ASCII_0508_071 byte        $70,$88,$80,$80,$B8,$88,$78,$00    ' -G-           
ASCII_0508_072 byte        $88,$88,$88,$F8,$88,$88,$88,$00    ' -H-           
ASCII_0508_073 byte        $70,$20,$20,$20,$20,$20,$70,$00    ' -I-           
ASCII_0508_074 byte        $38,$10,$10,$10,$10,$90,$60,$00    ' -J-           
ASCII_0508_075 byte        $88,$90,$A0,$C0,$A0,$90,$88,$00    ' -K-           
ASCII_0508_076 byte        $80,$80,$80,$80,$80,$80,$F8,$00    ' -L-           
ASCII_0508_077 byte        $88,$D8,$A8,$A8,$88,$88,$88,$00    ' -M-           
ASCII_0508_078 byte        $88,$88,$C8,$A8,$98,$88,$88,$00    ' -N-           
ASCII_0508_079 byte        $70,$88,$88,$88,$88,$88,$70,$00    ' -O-           
ASCII_0508_080 byte        $F0,$88,$88,$F0,$80,$80,$80,$00    ' -P-           
ASCII_0508_081 byte        $70,$88,$88,$88,$A8,$90,$68,$00    ' -Q-           
ASCII_0508_082 byte        $F0,$88,$88,$F0,$A0,$90,$88,$00    ' -R-           
ASCII_0508_083 byte        $70,$88,$80,$70,$08,$88,$70,$00    ' -S-           
ASCII_0508_084 byte        $F8,$20,$20,$20,$20,$20,$20,$00    ' -T-           
ASCII_0508_085 byte        $88,$88,$88,$88,$88,$88,$70,$00    ' -U-           
ASCII_0508_086 byte        $88,$88,$88,$88,$88,$50,$20,$00    ' -V-           
ASCII_0508_087 byte        $88,$88,$88,$A8,$A8,$D8,$88,$00    ' -W-           
ASCII_0508_088 byte        $88,$88,$50,$20,$50,$88,$88,$00    ' -X-           
ASCII_0508_089 byte        $88,$88,$50,$20,$20,$20,$20,$00    ' -Y-           
ASCII_0508_090 byte        $F8,$08,$10,$20,$40,$80,$F8,$00    ' -Z-           
ASCII_0508_091 byte        $F0,$C0,$C0,$C0,$C0,$C0,$F0,$00    ' -[-           
ASCII_0508_092 byte        $00,$80,$40,$20,$10,$08,$00,$00    ' -\-           
ASCII_0508_093 byte        $78,$18,$18,$18,$18,$18,$78,$00    ' -]-           
ASCII_0508_094 byte        $20,$70,$A8,$20,$20,$20,$20,$00    ' -^-           
ASCII_0508_095 byte        $00,$20,$40,$F8,$40,$20,$00,$00    ' -_-           
ASCII_0508_096 byte        $20,$10,$08,$00,$00,$00,$00,$00    ' -`-           
ASCII_0508_097 byte        $00,$00,$E0,$10,$70,$90,$68,$00    ' -a-           
ASCII_0508_098 byte        $80,$80,$B0,$C8,$88,$C8,$B0,$00    ' -b-           
ASCII_0508_099 byte        $00,$00,$70,$88,$80,$80,$70,$00    ' -c-           
ASCII_0508_100 byte        $08,$08,$68,$98,$88,$98,$68,$00    ' -d-           
ASCII_0508_101 byte        $00,$00,$70,$88,$F0,$80,$70,$00    ' -e-           
ASCII_0508_102 byte        $30,$48,$40,$F0,$40,$40,$40,$00    ' -f-           
ASCII_0508_103 byte        $00,$00,$70,$88,$88,$78,$08,$F0    ' -g-           
ASCII_0508_104 byte        $80,$80,$B0,$C8,$88,$88,$88,$00    ' -h-           
ASCII_0508_105 byte        $20,$00,$00,$20,$20,$20,$20,$00    ' -i-           
ASCII_0508_106 byte        $10,$00,$00,$30,$10,$10,$10,$60    ' -j-           
ASCII_0508_107 byte        $80,$80,$90,$A0,$C0,$A0,$98,$00    ' -k-           
ASCII_0508_108 byte        $60,$20,$20,$20,$20,$20,$70,$00    ' -l-           
ASCII_0508_109 byte        $00,$00,$50,$A8,$A8,$A8,$A8,$00    ' -m-           
ASCII_0508_110 byte        $00,$00,$B0,$48,$48,$48,$48,$00    ' -n-           
ASCII_0508_111 byte        $00,$00,$70,$88,$88,$88,$70,$00    ' -o-           
ASCII_0508_112 byte        $00,$00,$F0,$88,$88,$F0,$80,$80    ' -p-           
ASCII_0508_113 byte        $00,$00,$78,$88,$88,$78,$08,$08    ' -q-           
ASCII_0508_114 byte        $00,$00,$B0,$48,$40,$40,$40,$00    ' -r-           
ASCII_0508_115 byte        $00,$00,$78,$80,$70,$08,$F0,$00    ' -s-           
ASCII_0508_116 byte        $40,$40,$F8,$40,$40,$48,$30,$00    ' -t-           
ASCII_0508_117 byte        $00,$00,$90,$90,$90,$90,$68,$00    ' -u-           
ASCII_0508_118 byte        $00,$00,$88,$88,$88,$50,$20,$00    ' -v-           
ASCII_0508_119 byte        $00,$00,$A8,$A8,$A8,$A8,$50,$00    ' -w-           
ASCII_0508_120 byte        $00,$00,$88,$50,$20,$50,$88,$00    ' -x-           
ASCII_0508_121 byte        $00,$00,$88,$88,$98,$68,$08,$F0    ' -y-           
ASCII_0508_122 byte        $00,$00,$F8,$10,$20,$40,$F8,$00    ' -z-           
ASCII_0508_123 byte        $20,$40,$40,$80,$40,$40,$20,$00    ' -{-           
ASCII_0508_124 byte        $20,$20,$20,$00,$20,$20,$20,$00    ' -|-           
ASCII_0508_125 byte        $20,$10,$10,$08,$10,$10,$20,$00    ' -}-           
ASCII_0508_126 byte        $00,$00,$40,$A8,$10,$00,$00,$00    ' -~-           
ASCII_0508_127 byte        $A8,$50,$A8,$50,$A8,$50,$A8,$00    ' --

DAT scratchRomCharacter     byte 0[64]
 }
{DAT eyeNormalEz         byte %00000000
                        byte %01111100
                        byte %11111110
                        byte %11000110
                        byte %11000110
                        byte %11111110
                        byte %01111100
                        byte %00000000

normalPort              byte %00000000
                        byte %00111110
                        byte %01111111
                        byte %01100011
                        byte %01100011
                        byte %01111111
                        byte %00111110
                        byte %00000000
                        
eyeNormal2Ez            byte %00000000
                        byte %00111110
                        byte %01111111
                        byte %01100011
                        byte %01100011
                        byte %01111111
                        byte %00111110
                        byte %00000000

normal2Port             byte %00000000
                        byte %01111100
                        byte %11111110
                        byte %11000110
                        byte %11000110
                        byte %11111110
                        byte %01111100
                        byte %00000000
                        
eyeSad                  byte %00000000
                        byte %00000000
                        byte %11111110
                        byte %11111110
                        byte %11000110
                        byte %01111100
                        byte %00000000
                        byte %00000000

sadPort                 byte %00000000
                        byte %00000000
                        byte %01111111
                        byte %01111111
                        byte %01100011
                        byte %00111110
                        byte %00000000
                        byte %00000000

eyeSad2                 byte %00000000
                        byte %00000000
                        byte %01111111
                        byte %01111111
                        byte %01100011
                        byte %00111110
                        byte %00000000
                        byte %00000000

sad2Port                byte %00000000
                        byte %00000000
                        byte %11111110
                        byte %11111110
                        byte %11000110
                        byte %01111100
                        byte %00000000
                        byte %00000000

                        
eyeSurprise             byte %00111000           
                        byte %01111100
                        byte %11000110
                        byte %11000110
                        byte %11000110
                        byte %11000110
                        byte %01111100
                        byte %00111000           

surprisePort            byte %00011100           
                        byte %00111110
                        byte %01100011
                        byte %01100011
                        byte %01100011
                        byte %01100011
                        byte %00111110
                        byte %00011100           

eyeSurprise2            byte %00011100           
                        byte %00111110
                        byte %01100011
                        byte %01100011
                        byte %01100011
                        byte %01100011
                        byte %00111110
                        byte %00011100           

surprise2Port           byte %00111000           
                        byte %01111100
                        byte %11000110
                        byte %11000110
                        byte %11000110
                        byte %11000110
                        byte %01111100
                        byte %00111000           
                        
}
                        
DAT eyeNormal           byte %01111100
                        byte %11111110
                        byte %11000110
                        byte %11000110
                        byte %11111110
                        byte %01111100
                        byte %00000000
                        byte %00000000
                        
eyeSad                  byte %11111110
                        byte %11111110
                        byte %11000110
                        byte %01111100
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        
eyeSurprise             byte %00111000           
                        byte %01111100
                        byte %11000110
                        byte %11000110
                        byte %11000110
                        byte %11000110
                        byte %01111100
                        byte %00111000           
                        
eyeHappy                byte %00111000
                        byte %01111100
                        byte %11111110
                        byte %11000110
                        byte %11000110
                        byte %11111110
                        byte %01111100
                        byte %00000000

eyeFear                 byte %01111100
                        byte %11111110
                        byte %11000110
                        byte %11000110
                        byte %11000110
                        byte %01111100
                        byte %00000000
                        byte %00000000
                        
eyeCrossed              byte %00111100
                        byte %01111110
                        byte %11100110
                        byte %11100110
                        byte %11100110
                        byte %01111110
                        byte %00111100
                        byte %00000000

eyeOvalVert             byte %00111000
                        byte %01111100
                        byte %11111110
                        byte %11000110
                        byte %11000110
                        byte %11111110
                        byte %01111100
                        byte %00111000

eyeOvalHor              byte %00111100
                        byte %01111110
                        byte %11100111
                        byte %11100111
                        byte %11100111
                        byte %01111110
                        byte %00111100
                        byte %00000000
                        
eyeSlitVert0            byte %00110000
                        byte %01111000
                        byte %11111100
                        byte %11001100
                        byte %11001100
                        byte %11111100
                        byte %01111000
                        byte %00110000

eyeSlitVert1            byte %00100000
                        byte %01110000
                        byte %11111000
                        byte %11011000
                        byte %11011000
                        byte %11111000
                        byte %01110000
                        byte %00100000

eyeSlitHor0             byte %00111100
                        byte %01111110
                        byte %11100111
                        byte %11100111
                        byte %01111110
                        byte %00111100
                        byte %00000000
                        byte %00000000
                        
eyeSlitHor1             byte %00111100
                        byte %01111110
                        byte %11100111
                        byte %01111110
                        byte %00111100
                        byte %00000000
                        byte %00000000
                        byte %00000000

normalBlink0            byte %01111100
                        byte %11111110
                        byte %11000110
                        byte %11000110
                        byte %11111110
                        byte %01111100
                        byte %00000000
                        byte %00000000

normalBlink1            byte %00000000
                        byte %11111110
                        byte %11000110
                        byte %11000110
                        byte %11111110
                        byte %01111100
                        byte %00000000
                        byte %00000000

normalBlink2            byte %00000000
                        byte %00000000
                        byte %11000110
                        byte %11000110
                        byte %11111110
                        byte %01111100
                        byte %00000000
                        byte %00000000

normalBlink3            byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %11000110
                        byte %11111110
                        byte %01111100
                        byte %00000000
                        byte %00000000
                        
normalBlink4            byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %11111110
                        byte %01111100
                        byte %00000000
                        byte %00000000
                                                                         
normalBlink5            byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %01111100
                        byte %00000000
                        byte %00000000

normalBlink6            byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                 
sadBlink0               byte %11111110
                        byte %11111110
                        byte %11000110
                        byte %01111100
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000

sadBlink1               byte %00000000
                        byte %11111110
                        byte %11000110
                        byte %01111100
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000

sadBlink2               byte %00000000
                        byte %00000000
                        byte %11000110
                        byte %01111100
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000

sadBlink3               byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %01111100
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000

surpriseBlink0          byte %00111000           
                        byte %01111100
                        byte %11000110
                        byte %11000110
                        byte %11000110
                        byte %11000110
                        byte %01111100
                        byte %00111000
                        
surpriseBlink1          'byte %00000000
                        byte %00111000           
                        byte %01111100
                        byte %11000110
                        byte %11000110
                        byte %01111100
                        byte %00111000                         
                        byte %00000000
                        byte %00000000
                                 
                        
surpriseBlink2          'byte %00000000
                        'byte %00000000           
                        byte %01111100
                        byte %11000110
                        byte %11000110
                        byte %01111100                        
                        byte %00000000                        
                        byte %00000000                        
                        byte %00000000
                        byte %00000000           
                                                 
surpriseBlink3          'byte %00000000
                        'byte %00000000           
                        'byte %00000000           
                        byte %01111100
                        byte %01111100                        
                        byte %00000000           
                        byte %00000000                        
                        byte %00000000                        
                        byte %00000000
                        byte %00000000           
                        byte %00000000           

slitVertBlink0          byte %00110000
                        byte %01111000
                        byte %11111100
                        byte %11001100
                        byte %11001100
                        byte %11111100
                        byte %01111000
                        byte %00110000

slitVertBlink1          {byte %00110000
                        byte %01111000
                        byte %01111000
                        byte %01001000
                        byte %01001000
                        byte %01111000
                        byte %01111000
                        byte %00110000}
                        byte %01100000
                        byte %11110000
                        byte %11110000
                        byte %10010000
                        byte %10010000
                        byte %11110000
                        byte %11110000
                        byte %01100000

slitVertBlink2          {byte %00000000
                        byte %00110000
                        byte %00110000
                        byte %01001000
                        byte %01001000
                        byte %00110000
                        byte %00110000
                        byte %00000000}
                        byte %01100000
                        byte %01100000
                        byte %01100000
                        byte %10010000
                        byte %10010000
                        byte %01100000
                        byte %01100000
                        byte %01100000

slitVertBlink3          {byte %00000000
                        byte %00000000
                        byte %00100000
                        byte %00100000
                        byte %00100000
                        byte %00100000
                        byte %00000000
                        byte %00000000}
                        byte %10000000
                        byte %10000000
                        byte %10000000
                        byte %10000000
                        byte %10000000
                        byte %10000000
                        byte %10000000
                        byte %10000000
                        
{                        
DAT '' Eye Masks  {not presently used}
mskNormal               byte %01111100
                        byte %11111110
                        byte %11111110
                        byte %11111110
                        byte %11111110
                        byte %01111100
                        byte %00000000
                        byte %00000000
                        
mskSad                  byte %11111110
                        byte %11111110
                        byte %11111110
                        byte %01111100
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        
mskSurprise             byte %00111000           
                        byte %01111100
                        byte %11111110
                        byte %11111110
                        byte %11111110
                        byte %11111110
                        byte %01111100
                        byte %00111000           
                        
mskHappy                byte %00111000
                        byte %01111100
                        byte %11111110
                        byte %11111110
                        byte %11111110
                        byte %11111110
                        byte %01111100
                        byte %00000000

mskFear                 byte %01111100
                        byte %11111110
                        byte %11111110
                        byte %11111110
                        byte %11111110
                        byte %01111100
                        byte %00000000
                        byte %00000000
                        
mskCrossed              byte %00111100
                        byte %01111110
                        byte %11111110
                        byte %11111110
                        byte %11111110
                        byte %01111110
                        byte %00111100
                        byte %00000000

mskOvalVert             byte %00111000
                        byte %01111100
                        byte %11111110
                        byte %11111110
                        byte %11111110
                        byte %11111110
                        byte %01111100
                        byte %00111000

mskOvalHor              byte %00111100
                        byte %01111110
                        byte %11111111
                        byte %11111111
                        byte %11111111
                        byte %01111110
                        byte %00111100
                        byte %00000000
                        
mskSlitVert0            byte %00110000
                        byte %01111000
                        byte %11111100
                        byte %11111100
                        byte %11111100
                        byte %11111100
                        byte %01111000
                        byte %00110000

mskSlitVert1            byte %00100000
                        byte %01110000
                        byte %11111000
                        byte %11111000
                        byte %11111000
                        byte %11111000
                        byte %01110000
                        byte %00100000

mskSlitHor0             byte %00111100
                        byte %01111110
                        byte %11111111
                        byte %11111111
                        byte %01111110
                        byte %00111100
                        byte %00000000
                        byte %00000000
                        
mskSlitHor1             byte %00111100
                        byte %01111110
                        byte %11111111
                        byte %01111110
                        byte %00111100
                        byte %00000000
                        byte %00000000
                        byte %00000000

DAT
eyeCharacter            byte "@", ")", "!", "(", "#", "^", "{", "}", "|", "[", "_", "]", 0                        
eyeAlignFromOuter       byte 0[5], 1, 0, 1, 2, 0[2]
eyeAlignFromTop         byte 1, 2, 0, 0, 1, 0[5], 1, 1  }
{'' Not presently used.
eyePupilTop             byte 0[EYE_TYPES]
eyePupilBottom          byte 0[EYE_TYPES] 
eyePupilOuter           byte 0[EYE_TYPES]    
eyePupilInner           byte 0[EYE_TYPES]
eyeWidth                byte 0[EYE_TYPES]
eyeHeight               byte 0[EYE_TYPES]  }
    